@@ -28,6 +28,7 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -44,7 +45,9 @@ import static org.mockito.Mockito.when;
 
 import org.apache.commons.io.IOUtils;
 import org.apache.fontbox.cff.CFFParser;
+import org.apache.fontbox.ttf.GlyphData;
 import org.apache.fontbox.ttf.TTFParser;
+import org.apache.fontbox.ttf.TrueTypeFont;
 import org.apache.fontbox.type1.Type1Font;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSDictionary;
@@ -105,6 +108,8 @@ public class PDFBoxAdapterTestCase {
     protected static final String TTSubset5 = "ttsubset5.pdf";
     protected static final String TTSubset6 = "ttsubset6.pdf";
     protected static final String TTSubset7 = "ttsubset7.pdf";
+    protected static final String TTSubset8 = "ttsubset8.pdf";
+    protected static final String TTSubset9 = "ttsubset9.pdf";
     protected static final String CFFCID1 = "cffcid1.pdf";
     protected static final String CFFCID2 = "cffcid2.pdf";
     protected static final String Type1Subset1 = "t1subset.pdf";
@@ -139,7 +144,7 @@ public class PDFBoxAdapterTestCase {
         doc.setFormXObjectEnabled(formXObject);
         pdfpage.setDocument(doc);
         pdfpage.setObjectNumber(1);
-        return new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        return new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
     }
 
     protected static PDDocument load(String pdf) throws IOException {
@@ -258,7 +263,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage pdfpage = getPDFPage(pdfdoc);
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
         PDDocument doc = load(ANNOT);
         PDPage page = doc.getPage(0);
         AffineTransform at = new AffineTransform();
@@ -294,7 +299,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage pdfpage = getPDFPage(pdfdoc);
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
         PDDocument doc = load(ACCESSIBLERADIOBUTTONS);
         PDPage page = doc.getPage(0);
         AffineTransform at = new AffineTransform();
@@ -314,7 +319,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage pdfpage = getPDFPage(pdfdoc);
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
         PDDocument doc = load(ACCESSIBLERADIOBUTTONS);
         COSArray fields = (COSArray)
                 doc.getDocumentCatalog().getAcroForm().getCOSObject().getDictionaryObject(COSName.FIELDS);
@@ -336,7 +341,7 @@ public class PDFBoxAdapterTestCase {
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
         Map<Integer, PDFArray> pageNumbers = new HashMap<Integer, PDFArray>();
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), pageNumbers);
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), pageNumbers);
         PDDocument doc = load(LINK);
         PDPage page = doc.getPage(0);
         AffineTransform at = new AffineTransform();
@@ -358,7 +363,7 @@ public class PDFBoxAdapterTestCase {
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
         Map<Integer, PDFArray> pageNumbers = new HashMap<Integer, PDFArray>();
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), pageNumbers);
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), pageNumbers);
         PDDocument doc = load(XFORM);
         PDPage page = doc.getPage(0);
         AffineTransform at = new AffineTransform();
@@ -508,7 +513,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage pdfpage = getPDFPage(pdfdoc);
         pdfdoc.assignObjectNumber(pdfpage);
         pdfpage.setDocument(pdfdoc);
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
         PDDocument doc = load(src);
         PDPage page = doc.getPage(0);
         adapter.createStreamFromPDFBoxPage(doc, page, "key", new AffineTransform(), null, new Rectangle());
@@ -612,7 +617,9 @@ public class PDFBoxAdapterTestCase {
     @Test
     public void testErrorMsgToPDF() throws IOException {
         String msg = "";
-        PDFRenderingContext context = new PDFRenderingContext(null, null, null, null);
+        PDFDocument pdfdoc = new PDFDocument("");
+        PDFContentGenerator contentGenerator = new PDFContentGenerator(pdfdoc, null, null);
+        PDFRenderingContext context = new PDFRenderingContext(null, contentGenerator, null, null);
         ImagePDF imagePDF = new ImagePDF(new ImageInfo(ERROR, null), null);
         try {
             new PDFBoxImageHandler().handleImage(context, imagePDF, null);
@@ -688,7 +695,7 @@ public class PDFBoxAdapterTestCase {
         PDFPage pdfpage = getPDFPage(pdfdoc);
         pdfpage.setDocument(pdfdoc);
         pdfpage.setObjectNumber(1);
-        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap(), new HashMap<Integer, PDFArray>());
+        PDFBoxAdapter adapter = new PDFBoxAdapter(pdfpage, new HashMap<>(), new HashMap<Integer, PDFArray>());
         PDDocument doc = load(pdf);
         PDPage page = doc.getPage(0);
         AffineTransform at = new AffineTransform();
@@ -734,4 +741,25 @@ public class PDFBoxAdapterTestCase {
         String msg = writeText(fi, TTSubset7);
         Assert.assertTrue(msg, msg.contains("( )Tj"));
     }
+
+    @Test
+    public void testReorderGlyphs() throws IOException {
+        FontInfo fontInfo = new FontInfo();
+        writeText(fontInfo, TTSubset8);
+        writeText(fontInfo, TTSubset9);
+        List<Integer> compositeList = new ArrayList<>();
+        for (Typeface font : fontInfo.getUsedFonts().values()) {
+            InputStream inputStream = ((CustomFont) font).getInputStream();
+            TTFParser parser = new TTFParser(true);
+            TrueTypeFont trueTypeFont = parser.parse(inputStream);
+            int i = 0;
+            for (GlyphData glyphData : trueTypeFont.getGlyph().getGlyphs()) {
+                if (glyphData != null && glyphData.getDescription().isComposite()) {
+                    compositeList.add(i);
+                }
+                i++;
+            }
+        }
+        Assert.assertEquals(compositeList, Arrays.asList(18, 19, 39, 42, 62, 63, 29));
+    }
 }
